@@ -2,6 +2,7 @@ from typing import Dict, Union, Type
 
 import construct
 from construct import Construct, Probe, Struct, Adapter
+from construct.core import FocusedSeq, Byte, If, IfThenElse, Optional, Peek
 
 import mercury_engine_data_structures.dread_data
 from mercury_engine_data_structures.common_types import make_vector
@@ -23,16 +24,31 @@ def ConfirmType(name: str):
     )
 
 
+def _has_duplicated_keys(obj):
+    seen = set()
+    for item in obj:
+        if item.type in seen:
+            return True
+        seen.add(item.type)
+    return False
+
+
 class ObjectAdapter(Adapter):
     def _decode(self, obj: construct.ListContainer, context, path):
+        if _has_duplicated_keys(obj):
+            return obj
+
         result = construct.Container()
         for item in obj:
             if item.type in result:
                 raise construct.ConstructError(f"Type {item.type} found twice in object", path)
             result[item.type] = item.item
+
         return result
 
     def _encode(self, obj: construct.Container, context, path):
+        if isinstance(obj, construct.ListContainer):
+            return obj
         return construct.ListContainer(
             construct.Container(
                 type=type_,
@@ -47,7 +63,16 @@ def Object(fields: Dict[str, Union[Construct, Type[Construct]]], *,
     all_types = list(fields)
 
     fields = {
-        name: name / conn
+        name: FocusedSeq(
+            name,
+            "next_field" / Optional(Peek(PropertyEnum)),
+            "remaining" / Optional(Peek(Byte)),
+            name / IfThenElse(
+                construct.this._parsing,
+                If(lambda this: this.remaining is not None and (this.next_field is None or this.next_field not in fields.keys()), conn),
+                Optional(conn)
+            )
+        )
         for name, conn in fields.items()
     }
     for type_name in all_types:
